#include "graphics.h"
#include <SDL.h>

int clear_screen(SDL_Surface *screen) {
  SDL_FillRect(screen, NULL, SDL_MapRGB(screen->format, 255, 255, 255));
  
  return 0;
}

int show_pixel(SDL_Surface *screen, long x, long y, Uint32 color) {
  if(x < 0 || y < 0 || x > screen->w || y > screen->h)
    return -1; // BAAD coordinates
  
  Uint32 *pixel;
  
  pixel = (Uint32*) (((Uint8*) screen->pixels) + y*screen->pitch + 4*x);
  *pixel = color;
  
  return 0;
}

int show_segment(SDL_Surface *screen, long x1, long x2, long y, Uint32 color) {
  long t;
  
  if(x1 > x2) {
    t = x1;
    x1 = x2;
    x2 = t;
  }
   
  if(y < 0 || y > screen->h || x2 < 0 || x1 > screen->w)
    return -1; // BAAD coordinates
   
  if(x1 < 0)
    x1 = 0;
    
  if(x2 > screen->w)
    x2 = screen->w - 1;
    
  if(x1 == x2)
    return show_pixel(screen, x1, y, color);
  
  Uint32 *pixel;

  pixel = (Uint32*) (((Uint8*) screen->pixels) + y*screen->pitch + 4*x1);
  while(x1++ <= x2) {
    *pixel++ = color;
  }
  
  return 0;
}

int show_rect(SDL_Surface *screen, long x, long y, long w, long h, Uint32 color) {
  if(x < 0 || y < 0 || x > screen->w || y > screen->h)
    return -1; // BAAD coordinates
  
  SDL_Rect rect;
  rect.x = x;
  rect.y = y;
  rect.w = w;
  rect.h = h;
  
  SDL_FillRect(screen, &rect, color);

  return 0;
}
