#include "const.h"
#include "rasterizer.h"
#include <SDL.h>

int main (int argc, char *argv[])
{
  SDL_Surface *screen;
  
  if(SDL_Init(SDL_INIT_VIDEO) == -1) {
    printf("SDL Init Error: %s\n", SDL_GetError());
    return 1;
  }
  
  if(!(screen = SDL_SetVideoMode(DIM_H, DIM_V, 32, SDL_SWSURFACE))) {
    printf("Screen Init Error: %s\n", SDL_GetError());
  }
  
  // prepare the control points for demonstration
  control_point points[12];
  
  points[0] = make_control_point(150, 10, TRUE);

  points[1] = make_control_point(225, 10, FALSE);
  points[0].next = points+1;
  points[2] = make_control_point(290, 75, FALSE);
  points[1].next = points+2;

  points[3] = make_control_point(290, 150, TRUE);
  points[2].next = points+3;
  
  points[4] = make_control_point(290, 225, FALSE);
  points[3].next = points+4;
  points[5] = make_control_point(225, 290, FALSE);
  points[4].next = points+5;
  
  points[6] = make_control_point(150, 290, TRUE);
  points[5].next = points+6;
  
  points[7] = make_control_point(75, 290, FALSE);
  points[6].next = points+7;
  points[8] = make_control_point(10, 225, FALSE);
  points[7].next = points+8;
  
  points[9] = make_control_point(10, 150, TRUE);
  points[8].next = points+9;
  
  points[10] = make_control_point(10, 75, FALSE);
  points[9].next = points+10;
  points[11] = make_control_point(75, 10, FALSE);
  points[10].next = points+11;
  
  points[11].next = points; // close the list
    
  rasterizer *my_ras = rasterizer_init(4096);
    
  SDL_Event event;
  control_point *dragged_point = NULL;
  unsigned char needs_display = 1; 
  
  // start event loop
  
  while(SDL_WaitEvent(&event)) {
    if(event.type == SDL_QUIT)  
      break;
    
    if((event.type == SDL_MOUSEMOTION) && dragged_point) {
      if(event.motion.x > CP_DIAM && event.motion.y > CP_DIAM && 
         event.motion.x < DIM_H - CP_DIAM && event.motion.y < DIM_V - CP_DIAM)
        dragged_point->pos = make_point(event.motion.x, event.motion.y);
      needs_display = 1;
    }
      
    if(event.type == SDL_MOUSEBUTTONDOWN) {
      dragged_point = hit_test(points, event.button.x, event.button.y);
    }
    
    if(event.type == SDL_MOUSEBUTTONUP) {
      dragged_point = NULL;
    }

    if(needs_display) {
      clear_screen(screen);
      reset(my_ras);
    
      path(my_ras, points);
      render(my_ras, screen, SDL_MapRGBA(screen->format, 0, 50, 255, 1));
    
      draw_control_points(screen, points, SDL_MapRGBA(screen->format, 255, 0, 0, 1));
    
      SDL_UpdateRect(screen, 0, 0, 0, 0);
      
      needs_display = 0;
    }
  }
  
  SDL_Quit();
  return 0;
}


