#ifndef RASTERIZER_H
#define RASTERIZER_H

#include "primitives.h" 
#include "controls.h"

#define MAX_CURVES 32

typedef enum _monotony {
  dir_up,
  dir_down,
  dir_flat,
  dir_none
} monotony;

typedef struct _edge{
  long y_min;
  long y_max;
  monotony direction;
  
  // pixel data of the edge
  long *pixels;
  long pixels_length;
  
  struct _edge *next;
} edge;

typedef struct {
  edge *inactive_edges;
  edge *left_edges;
  edge *right_edges;
  
  void *edge_buffer; // edges pixel data
  void *edge_buffer_top;
  edge *top_edge;
  
  // stack of curves for rasterization
  point curve_stack[1 + 3*MAX_CURVES]; // simple array of on-curve and control points
  point *curve_stack_top;
  point *top_curve;
  
  point current_point;
  point current_path_start;
} rasterizer;

rasterizer* rasterizer_init(long edge_data_size);
void reset(rasterizer *self);

int path(rasterizer *self, control_point *points);

// ALWAYS define the path clockwise unless you're defining a "hole"
void move_to(rasterizer *self, point p0);
void curve_to(rasterizer *self, point c0, point c1, point p1);
void line_to(rasterizer *self, point p1);

void render(rasterizer *self, SDL_Surface *screen, Uint32 color);

// privte

void rasterize_curves(rasterizer *self);  
void split_curve(rasterizer *self);

void start_edge(rasterizer *self, ffloat y1, ffloat y2, monotony direction);

void sort_inactive_edges(rasterizer *self);

void init_scanline(rasterizer *self, long scanline); 
void sort_edge_lists(rasterizer *self, long scanline);
void drop_finished_edges(rasterizer *self, long scanline);

monotony curve_monotony(point p0, point c0, point c1, point p1);
int too_wide(point *curve);
int sl_cross_count(point *curve);

#endif
